class DefaultRecordDefinition:
  """
  Provides default values for the constants for the definition of record types.
  """
  RECORD_TYPE = None
  POSFIELDS = []
  PREDEFINED_TAGS = []
  DATATYPE = {}
  NAME_FIELD = None
  STORAGE_KEY = None
  FIELD_ALIAS = None
  REFERENCE_FIELDS = []
  BACKREFERENCE_RELATED_FIELDS = []
  DEPENDENT_LINES = []
  OTHER_REFERENCES = []
  REFERENCE_INITIALIZERS = None

